from abc import ABC

from ..classes.coordinate import Coordinate

class Actor(ABC):
    """
    Abstract base class to act as basis for all actors on the game board
    """
    def __init__(self, p_frames_per_second: int, p_coordinate: Coordinate):
        pass

    def sleep(self):
        pass

    def eat(self):
        pass

    def is_alive(self):
        pass

    def die(self):
        pass

    def get_position(self):
        pass

    def get_health(self):
        pass

    def get_state(self):
        pass

    def get_energy(self):
        pass

    def move(self):
        pass

    def run_turn(self):
        pass

    def draw(self):
        pass



from abc import ABC

from pygame import Surface

from ..classes.coordinate import Coordinate

class Consumable(ABC):
    """
    Abstract base class to act as a basis for food consumables 
    """

    def __init__(self):
        pass

    def get_consumable_value(self):
        pass

    def get_consumable_name(self):
        pass

    def get_consumable_weight(self):
        pass

    def draw(self, p_game_screen: Surface, p_coordinate: Coordinate):
        pass



from ..abstract.consumable import Consumable

from ..classes.food import Pear, Apple, Strawberry

class ConsumableFactory:
    """
    A factory for generating Consumables
    """

    def get_consumable(self, p_consumable_name: str) -> Consumable:
        """
        Factory function for generating new consumables given the consumable name

        Args:
            p_consumable_name: A name that matches the name of a Consumable

        Returns:
            The new consumable

        Raises:
            ValueError: If no class name matches the p_consumable_name
        """
        if p_consumable_name.lower() == "pear":
            return Pear()
        elif p_consumable_name.lower() == "apple":
            return Apple()
        elif p_consumable_name.lower() == "strawberry":
            return Strawberry()
        else:
            raise ValueError(p_consumable_name)



import enum


class Age(enum.Enum):
    """
    Enum to represent the age state
    """
    YOUNG = "YOUNG"
    TEENAGE = "TEENAGE"
    MIDDLEAGE = "MIDDLEAGE"
    ELDERLY = "ELDERLY"
    DECEASED = "DECEASED"



from abc import ABC
from pygame import color, font
from pygame.surface import Surface

from .actor import Actor

from ..classes.coordinate import Coordinate

class StatusElement(ABC):
    """
    Abstract base class to act as a basis for drawable screen elements
    """
    def __init__(self, p_coordinate: Coordinate):
        """

        """
        self.coordinate = p_coordinate

    def draw(self, p_game_screen: Surface, p_actor: Actor):
        pass

    def draw_header(self, p_game_screen: Surface, p_header: str):
        """
        Draws a header text over the Status elements Coordinate
        """
        largeText = font.Font('freesansbold.ttf',18)
        text_surface = largeText.render(p_header, True, color.THECOLORS['black'])
        text_rectangle = text_surface.get_rect()
        text_rectangle.center = (self.coordinate.get_x() + 25, self.coordinate.get_y() - 20)
        p_game_screen.blit(text_surface, text_rectangle)


from ..enums.direction import Direction


class Coordinate:
    """
    A class to represent a position on an X/Y screen
    """

    def __init__(self, x_position: int, y_position: int):
        """
        Create a new Coordinate

        Args:
            x_position: The int value representing the position on a x-axis
            y_position: The int value representing the position on a y-axis

        Returns:
            A new Coordinate
        """
        self.x_position = x_position
        self.y_position = y_position
    
    def get_coordinate(self) -> tuple:
        """
        Get a tuple representation of the coordinate object

        Returns:
            Tuple containing the x and y axis positions
        """
        return (self.x_position, self.y_position)

    def get_x(self) -> int:
        """
        Get the x position of the coordinate

        Returns:
            Int representing the position on the x axis
        """
        return self.x_position

    def get_y(self) -> int:
        """
        Get the y position of the coordinate

        Returns:
            Int representing the position on the y axis
        """
        return self.y_position



from pygame import draw, color, Surface
from pygame.locals import Rect

from ..abstract.status_element import StatusElement
from ..abstract.actor import Actor

from .coordinate import Coordinate

class HealthBar(StatusElement):
    """
    Simple status element class to display actor health
    """

    def draw(self, p_game_screen: Surface, p_actor: Actor):
        """
        Draw a health bar on a Surface using the health value of an Actor

        Args:
            p_game_screen: The Surface to draw to
            p_actor: The Actor to display the health value of
        """
        if p_actor.get_health() > 70:
            rectangle = Rect(self.coordinate.get_x(), self.coordinate.get_y(), 60, 20)
            draw.rect(p_game_screen, color.THECOLORS['green'], rectangle)
        elif p_actor.get_health() > 30:
            rectangle = Rect(self.coordinate.get_x(), self.coordinate.get_y(), 60, 20)
            draw.rect(p_game_screen, color.THECOLORS['orange'], rectangle)
        else:
            rectangle = Rect(self.coordinate.get_x(), self.coordinate.get_y(), 60, 20)
            draw.rect(p_game_screen, color.THECOLORS['red'], rectangle)
        
        rectangle_border = Rect(self.coordinate.get_x() - 3, self.coordinate.get_y() - 3, 65, 25)
        draw.rect(p_game_screen, color.THECOLORS['black'], rectangle_border, 2)

        self.draw_header(p_game_screen, "Health")


class EnergyBar(StatusElement):
    """
    Simple status element class to display actor energy
    """

    def draw(self, p_game_screen: Surface, p_actor: Actor):
        """
        Draw an Energy bar on a Surface using the energy value of an Actor

        Args:
            p_game_screen: The Surface to draw to
            p_actor: The Actor to display the energy status of
        """
        if p_actor.get_energy() > 70:
            rectangle = Rect(self.coordinate.get_x(), self.coordinate.get_y(), 60, 20)
            draw.rect(p_game_screen, color.THECOLORS['green'], rectangle)
        elif p_actor.get_energy() > 30:
            rectangle = Rect(self.coordinate.get_x(), self.coordinate.get_y(), 60, 20)
            draw.rect(p_game_screen, color.THECOLORS['orange'], rectangle)
        else:
            rectangle = Rect(self.coordinate.get_x(), self.coordinate.get_y(), 60, 20)
            draw.rect(p_game_screen, color.THECOLORS['red'], rectangle)
        
        rectangle_border = Rect(self.coordinate.get_x() - 3, self.coordinate.get_y() - 3, 65, 25)
        draw.rect(p_game_screen, color.THECOLORS['black'], rectangle_border, 2)

        self.draw_header(p_game_screen, "Energy")


from ..abstract.actor import Actor
from ..abstract.consumable import Consumable

from .coordinate import Coordinate

from ..enums.actor_state import ActorState
from ..enums.age import Age

from pygame import image, transform, Surface

from pkg_resources import resource_listdir, resource_filename, resource_string

class Pet(Actor):
    """
    The main pet of the game
    """

    def __init__(self, p_frames_per_second: int, p_coordinate: Coordinate):
        """
        Create a new Pet

        Args:
            p_frames_per_second: The frames per second that are run for the game
            p_coordinate: The starting position for the Pet

        Returns:
            The new Pet
        """
        self.energy = 100.0
        self.frames_per_second = p_frames_per_second
        self.age = 0.0
        self.fullness = 0.0
        self.animation_state_max_timer = 0
        self.state_timer = 0
        self.max_health = 100.0
        self.health = self.max_health
        self.position = p_coordinate
        self.state = ActorState.AWAKE

        self.image_width = 100
        self.image_height = 100
        self.image = self.generate_current_image()

    def run_turn(self):
        """
        The actor will operate its turn on the game board
        """
        if self.is_alive():
            self.health -= self.max_health / (60 * self.frames_per_second)  # In 60 seconds the pet will die from lack of nutrition, must factor in frame cycle
            self.age += (100 / (self.frames_per_second * 120))  # In 2 minutes it will get old and die
            self.energy -= 100 / (20 * self.frames_per_second)  # In twenty seconds it will run out of energy and go to sleep
            
            if self.fullness >= 70:
                self.poop()

            if self.state_timer > self.animation_state_max_timer * self.frames_per_second:  # In 3 seconds revert to normal state
                self.awake()
            else:
                self.state_timer += 1

            if self.energy <= 20 and not self.is_doing_action():
                self.sleep()

            if self.health <= 0 or self.age >= 100:
                self.die()

    def is_alive(self) -> bool:
        """
        Returns if the pet is dead

        Returns:
            Boolean representation of the pets alive status
        """
        if self.get_state() == ActorState.DEAD:
            return False
        else:
            return True
    
    def awake(self):
        """
        Set the pet to the awake state
        """
        if self.is_alive():
            self.image = self.generate_current_image()
            self.set_state(ActorState.AWAKE)
    
    def is_doing_action(self) -> bool:
        """
        Check if pet is currently doing an action
        """
        return not self.get_state() == ActorState.AWAKE

    def eat(self, p_consumable: Consumable):
        """
        The pet will eat the food its given to replenish health and increase fullness

        Args:
            p_consumable: A Consumable to be eaten by the Pet
        """
        if self.is_alive():
            if self.health + p_consumable.get_consumable_value() > self.max_health:
                self.health = self.max_health
            else:
                self.health += p_consumable.get_consumable_value()
            
            self.set_state(ActorState.EATING)
            self.fullness += p_consumable.get_consumable_weight()
            self.animation_state_max_timer = 2
            self.state_timer = 0

    def sleep(self):
        """
        The pet will go asleep to regain its energy
        """
        if self.is_alive():
            self.set_state(ActorState.ASLEEP)
            self.energy = 100
            self.state_timer = 0
            self.animation_state_max_timer = 5
            self.image = transform.scale(image.load(
                resource_filename('tamagotchi_tandersen', 'assets/fox-sleeping.png')), (self.image_width, self.image_height))

    def die(self):
        """
        Set the pets state to Dead
        """
        self.set_state(ActorState.DEAD)

    def get_position(self) -> tuple:
        """
        Get the coordinate tuple representation
        """
        return self.position.get_coordinate()
    
    def get_health(self) -> float:
        """
        Get the health of the pet
        """
        return self.health

    def get_state(self) -> ActorState:
        """
        Get the current ActorState of the pet
        """
        return self.state
    
    def get_age(self) -> Age:
        """
        Get an Age state representation of the pets age
        """
        if not self.is_alive():
            return Age.DECEASED
        elif self.age < 13:
            return Age.YOUNG
        elif self.age < 20:
            return Age.TEENAGE
        elif self.age < 65:
            return Age.MIDDLEAGE
        elif self.age <= 100:
            return Age.ELDERLY
        else:
            return Age.DECEASED
    
    def get_energy(self) -> float:
        """
        Get the energy level of the Pet
        """
        return self.energy

    def set_state(self, p_state: ActorState):
        """
        Set the state of the pet

        Args:
            p_state: The ActorState to change the Pet to
        """
        self.state = p_state

    def move(self):
        """
        @TODO add movement and animation to the Pet
        """
        return

    def draw(self, p_game_screen: Surface):
        """
        Draw the current representation of the pet on a Surface

        Args:
            p_game_screen: The Surface to draw the Pet on
        """
        p_game_screen.blit(
                            self.image,
                            (
                                self.position.get_x() - (self.image_width / 2),
                                self.position.get_y() - (self.image_height / 2)
                            )
                        )
    
    def poop(self):
        """
        Clear out the pets full stomach
        """
        if self.is_alive():
            self.set_state(ActorState.POOPING)
            self.fullness = 0
            self.state_timer = 0
            self.animation_state_max_timer = 3

    def generate_current_image(self) -> Surface:
        """
        To visually represent the state of the pet in its age cycle, this will return an image matching its age

        Returns:
            A Surface containing the image to display for the Pet
        """
        current_age = self.get_age()
        
        if current_age == Age.MIDDLEAGE:
            return transform.scale(image.load(
                resource_filename('tamagotchi_tandersen', 'assets/fox-middle.png')), (self.image_width, self.image_height))
        elif current_age == Age.ELDERLY:
            return transform.scale(image.load(
                resource_filename('tamagotchi_tandersen', 'assets/fox-old.png')), (self.image_width, self.image_height))
        elif current_age == Age.DECEASED:
            return transform.scale(image.load(
                resource_filename('tamagotchi_tandersen', 'assets/fox-sleeping.png')), (self.image_width, self.image_height))
        else:
            return transform.scale(image.load(
                resource_filename('tamagotchi_tandersen', 'assets/cute-fox.png')), (self.image_width, self.image_height))



"""
    This is my attempt at the Tamagotchi challenge
    Author: Tristan Andersen
"""

from .classes.game_board import GameBoard


if __name__ == "__main__":
    game_screen_size = (600, 400)

    game_board = GameBoard(game_screen_size[0], game_screen_size[1])

    game_board.run_game()  # Game loop runs in here, will exit upon closing



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tamagotchi_tandersen", # Replace with your own username
    version="1.0.0",
    author="Tristan Andersen",
    description="My attempt at the Tamagotchi challenge",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Tristan-Andersen/Tamagotchi-challenge",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={'': ['*.jpg'],
    '': ['*.png']
    },
    install_requires=['pygame'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


from ..abstract.consumable import Consumable

from .coordinate import Coordinate

from pygame import image, transform, Surface

from pkg_resources import resource_filename

class Pear(Consumable):
    """
    A Pear Consumable
    """

    def __init__(self):
        """
        Create a new Pear

        Returns:
            A new Pear
        """
        self.image_width = 30
        self.image_height = 30

        image_to_use = image.load(resource_filename('tamagotchi_tandersen', 'assets/pixel-pear.png'))
        self.image = transform.scale(image_to_use, (self.image_width, self.image_height))

    def get_consumable_value(self) -> int:
        """
        Get the value of the Pear

        Returns:
            The integer value of the Pear
        """
        return 40

    def get_consumable_name(self) -> str:
        """
        Get string representation of the Pear

        Returns:
            The string of Pear
        """
        return "Pear"
    
    def get_consumable_weight(self) -> int:
        """
        Get the weight of the Pear

        Returns:
            Weight of a Pear
        """
        return 50

    def draw(self, p_game_screen: Surface, p_coordinate: Coordinate):
        """
        Draw the Pear on a Surface given a Coordinate
        """
        p_game_screen.blit(
                            self.image,
                            (
                                p_coordinate.get_x() - (self.image_width / 2),
                                p_coordinate.get_y() - (self.image_height / 2)
                            )
                        )


class Apple(Consumable):
    """
    An Apple Consumable
    """

    def __init__(self):
        """
        Create a new Apple

        Returns:
            A new Apple
        """
        self.image_width = 30
        self.image_height = 30

        image_to_use = image.load(resource_filename('tamagotchi_tandersen', 'assets/pixel-apple.jpg'))
        self.image = transform.scale(image_to_use, (self.image_width, self.image_height))

    def get_consumable_value(self) -> int:
        """
        Get the value of the Apple

        Returns:
            The integer value of the Apple
        """
        return 60

    def get_consumable_name(self) -> str:
        """
        Get string representation of the Apple

        Returns:
            The string of Apple
        """
        return "Apple"

    def get_consumable_weight(self) -> int:
        """
        Get the weight of the Apple

        Returns:
            Weight of a Apple
        """
        return 70
    
    def draw(self, p_game_screen: Surface, p_coordinate: Coordinate):
        """
        Draw an Apple on a surface given a Coordinate
        """
        p_game_screen.blit(
                            self.image,
                            (
                                p_coordinate.get_x() - (self.image_width / 2),
                                p_coordinate.get_y() - (self.image_height / 2)
                            )
                        )


class Strawberry(Consumable):
    """
    A Strawberry consumable
    """

    def __init__(self):
        """
        Create a new Strawberry

        Returns:
            A new Strawberry
        """
        self.image_width = 30
        self.image_height = 30

        image_to_use = image.load(resource_filename('tamagotchi_tandersen', 'assets/pixel-strawberry.jpg'))
        self.image = transform.scale(image_to_use, (self.image_width, self.image_height))

    def get_consumable_value(self) -> int:
        """
        Get the value of the Strawberry

        Returns:
            The integer value of the Strawberry
        """
        return 20
    
    def get_consumable_name(self) -> str:
        """
        Get string representation of the Strawberry

        Returns:
            The string of Strawberry
        """
        return "Strawberry"

    def get_consumable_weight(self) -> int:
        """
        The weight value of the Strawberry

        Returns:
            Weight of a Strawberry
        """
        return 30
    
    def draw(self, p_game_screen: Surface, p_coordinate: Coordinate):
        """
        Draw the Strawberry on a Surface given a Coordinate
        """
        p_game_screen.blit(
                            self.image,
                            (
                                p_coordinate.get_x() - (self.image_width / 2),
                                p_coordinate.get_y() - (self.image_height / 2)
                            )
                        )



import pygame

from pygame.surface import Surface

# from . import FoodInventory, Coordinate, Pet

from .pet import Pet
from .food_inventory import FoodInventory
from .coordinate import Coordinate

from ..abstract.actor import Actor
from ..abstract.consumable import Consumable
from ..abstract.status_element import StatusElement

from ..enums.direction import Direction
from ..enums.actor_state import ActorState

from ..factory.consumable_factory import ConsumableFactory
from ..factory.status_bar_factory import StatusBarFactory

class GameBoard:
    """
    The operator and screen of the Tamagotchi challenge game
    """

    def __init__(self, x_length: int, y_length: int):
        """
        Generate a new GameBoard given the size of the screen

        Args:
            x_length: The x dimension size of the scren
            y_length: The y dimension size of the screen

        Returns:
            The new GameBoard
        """
        self.x_length = x_length
        self.y_length = y_length
        self.game_frames_per_second = 30

        self.actor = Pet(self.game_frames_per_second,
                         Coordinate(x_length / 2, y_length / 2)
                        )

        consumable_factory = ConsumableFactory()
        status_bar_factory = StatusBarFactory()

        food_to_create = ["pear", "apple", "strawberry", "pear", "strawberry", "strawberry"]
        food_inventory_list = []
        for food_name in food_to_create:
            try:
                food_inventory_list.append(consumable_factory.get_consumable(food_name))
            except ValueError as e:
                print("ValueError : {} is not an acceptable Consumable name".format(e))

        self.food_inventory = FoodInventory(food_inventory_list, Coordinate(500, 320))

        self.status_bars = [status_bar_factory.get_status_bar("health", Coordinate(40, 350)),
                            status_bar_factory.get_status_bar("energy", Coordinate(120, 350))]

        self.game_over = False
    
    def draw_header(self, p_game_screen: Surface, p_coordinate: Coordinate, p_header: str):
        """
        Draw large text on the game screen at a given coordinate

        Args:
            p_game_screen: The screen to display the text on
            p_coordinate: The coordinate to display the text at
            p_header: The large text to display
        """
        largeText = pygame.font.Font('freesansbold.ttf', 30)
        text_surface = largeText.render(p_header, True, pygame.color.THECOLORS['black'])
        text_rectangle = text_surface.get_rect()
        text_rectangle.center = (p_coordinate.get_x(), p_coordinate.get_y())
        p_game_screen.blit(text_surface, text_rectangle)

    def draw_normal_text(self, p_game_screen: Surface, p_coordinate: Coordinate, p_text: str):
        """
        Draw text on the game screen at a given coordinate

        Args:
            p_game_screen: The screen to display the text on
            p_coordinate: The coordinate to display the text at
            p_text: The text to display
        """
        largeText = pygame.font.Font('freesansbold.ttf', 18)
        text_surface = largeText.render(p_text, True, pygame.color.THECOLORS['black'])
        text_rectangle = text_surface.get_rect()
        text_rectangle.center = (p_coordinate.get_x(), p_coordinate.get_y())
        p_game_screen.blit(text_surface, text_rectangle)

    def get_actor(self) -> Actor:
        """
        Get the actor run by the game board

        Returns:
            The actor in play
        """
        return self.actor

    def get_board_size(self) -> tuple:
        """
        Get a tuple representation of the size of the game screen

        Returns:
            The tuple of the screen size
        """
        return (self.x_length, self.y_length)

    def get_game_over(self) -> bool:
        """
        Get the state of if the game is over

        Returns:
            The game over boolean
        """
        return self.game_over

    def run_event_queue(self):
        """
        The game loop event handler
        """
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.game_over = True
            if event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    self.food_inventory.change_active_food(Direction.LEFT)
                elif event.key == pygame.K_RIGHT:
                    self.food_inventory.change_active_food(Direction.RIGHT)
                elif event.key == pygame.K_e:
                    if not self.actor.get_state() == ActorState.ASLEEP:
                        food_to_eat = self.food_inventory.eat_food()
                        self.actor.eat(food_to_eat)
                elif event.key == pygame.K_s:
                    self.actor.sleep()
    
    def draw_instructions(self, p_game_screen: Surface):
        """
        Draws the instructions for the player on the Surface

        Args:
            p_game_screen: The Surface to draw the instructions on
        """
        self.draw_header(p_game_screen, Coordinate(self.x_length - 150, 30), "Controls")
        self.draw_normal_text(p_game_screen, Coordinate(self.x_length - 150, 50), "Press e to feed selected food")
        self.draw_normal_text(p_game_screen, Coordinate(self.x_length - 150, 70), "Press s to put pet to sleep")
        self.draw_normal_text(p_game_screen, Coordinate(self.x_length - 150, 90), "Use Left/Right keys to select food")

    def run_game(self):
        """
        Will run the game logic in a loop until the game is exited via closing the game window
        """
        pygame.init()

        screen = pygame.display.set_mode(self.get_board_size())

        pygame.display.set_caption('Tamagotchi Challenge')

        clock = pygame.time.Clock()

        while not self.get_game_over():
            if not self.actor.is_alive():
                break

            self.run_event_queue()

            screen.fill(pygame.color.THECOLORS['white'])
            self.actor.run_turn()
            self.actor.draw(screen)
            self.food_inventory.draw(screen)

            for status_bar in self.status_bars:
                status_bar.draw(screen, self.actor)
            
            self.draw_header(screen, Coordinate(70, 30), self.actor.get_state().value)
            self.draw_instructions(screen)

            pygame.display.update()
            clock.tick(self.game_frames_per_second)

        while not self.get_game_over():
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.game_over = True

            screen.fill(pygame.color.THECOLORS['white'])
            self.draw_header(screen, Coordinate(self.x_length/2, self.y_length/2 - 50), "Unfortunately, the Pet has died.")
            self.draw_header(screen, Coordinate(self.x_length/2, self.y_length/2), "But thanks for playing!")
            pygame.display.update()
            clock.tick(self.game_frames_per_second)
    
        pygame.quit()
        quit()



from ..abstract.consumable import Consumable

from ..classes.coordinate import Coordinate

from ..enums.direction import Direction

from pygame import draw, color, Surface
from pygame.locals import Rect


class FoodInventory:
    """
    Will contain the available Consumables for use by the actor
    """

    def __init__(self, p_food_list: list, p_coordinate: Coordinate):
        """
        Create a new FoodInventory

        Args:
            p_food_list: The list of food
            p_coordinate: The position for the FoodInventory to display at

        Returns:
            The new FoodInventory
        """
        self.food_list = p_food_list
        self.active_food_index = 0
        self.coordinate = p_coordinate

    def get_available_food(self) -> dict:
        """
        Returns a dictionary of foods available to consume

        Returns:
            A dict with consumable names as keys and occurences in food list as value
        """
        food_occurence_dictionary = {}

        for food in self.food_list:
            if food.get_consumable_name() in food_occurence_dictionary.keys():
                food_occurence_dictionary[food.get_consumable_name()] += 1
            else:
                food_occurence_dictionary[food.get_consumable_name()] = 1

        return food_occurence_dictionary

    def eat_food(self) -> Consumable:
        """
        Eat the current active food

        Returns:
            A Consumable object pop'd from the food_list
        """
        if len(self.food_list) == 0:
            return None
        else:
            tmp_food_index = self.active_food_index
            if self.active_food_index == len(self.food_list) - 1:
                self.active_food_index = 0

            return self.food_list.pop(tmp_food_index)

    def add_food(self, p_food: Consumable):
        """
        Add a food item to the list of available food

        Args:
            p_food: The Consumable to add to the list
        """
        self.food_list.append(p_food)

    def draw(self, p_game_screen: Surface):
        """
        Draws a box showing active food, with quantity with arrows to see the other foods

        Args:
            p_game_screen: The Surface to draw on
        """

        rectangle_size = (50, 50)

        rectangle_border = Rect(self.coordinate.get_coordinate(), rectangle_size)
        draw.rect(p_game_screen, color.THECOLORS['black'], rectangle_border, 5)

        if len(self.food_list) > 1:
            self.draw_arrows(p_game_screen, self.coordinate, rectangle_size)

        if len(self.food_list) > 0:
            self.food_list[self.active_food_index].draw(p_game_screen,
                                                        Coordinate(self.coordinate.get_x() + (rectangle_size[0]/2),
                                                                   self.coordinate.get_y() + (rectangle_size[1]/2))
                                                        )

    def draw_arrows(self, p_game_screen: Surface, p_coordinate: Coordinate, p_rect_size: tuple):
        """
        Internal function for drawing arrows either side of a rectangle

        Args:
            p_game_screen: Surface to draw the arrows to
            p_coordinate: The coordinate to center the arrows off
            p_rect_size: The size of the rectangle they are centered off
        """
        draw.polygon(p_game_screen,
                     color.THECOLORS['black'],
                     (
                        (p_coordinate.get_x() + p_rect_size[0] + 10, p_coordinate.get_y()),
                        (p_coordinate.get_x() + p_rect_size[0] + 30, p_coordinate.get_y() + (p_rect_size[1] / 2)),
                        (p_coordinate.get_x() + p_rect_size[0] + 10, p_coordinate.get_y() + (p_rect_size[1]))
                     )
                    )
        
        draw.polygon(p_game_screen,
                     color.THECOLORS['black'],
                     (
                        (p_coordinate.get_x() - 10, p_coordinate.get_y()),
                        (p_coordinate.get_x() - 30, p_coordinate.get_y() + (p_rect_size[1] / 2)),
                        (p_coordinate.get_x() - 10, p_coordinate.get_y() + (p_rect_size[1]))
                     )
                    )

    def change_active_food(self, direction: Direction):
        """
        Shift the active food to be consumed by giving a direction
        LEFT, DOWN to deincrement
        RIGHT, UP to increment

        Args:
            direction: The direction of the key press
        """
        if direction == Direction.LEFT :
            if self.active_food_index == 0:
                self.active_food_index = len(self.food_list) - 1
            else:
                self.active_food_index -= 1
        elif direction == Direction.RIGHT:
            if self.active_food_index == len(self.food_list) - 1:
                self.active_food_index = 0
            else:
                self.active_food_index += 1



import enum


class ActorState(enum.Enum):
    """
    Enum to represent the states an actor may be in
    """
    AWAKE = "AWAKE"
    ASLEEP = "ASLEEP"
    DEAD = "DEAD"
    POOPING = "POOPING"
    EATING = "EATING"
    WALKING = "WALKING"



from ..abstract.status_element import StatusElement

from ..classes.state_bars import HealthBar, EnergyBar
from ..classes.coordinate import Coordinate


class StatusBarFactory:
    """
    A factory for generating StatusElements
    """

    def get_status_bar(self, p_status_type: str, p_coordinate: Coordinate) -> StatusElement:
        """
        Factory function for generating new status bars given the status bar name

        Args:
            p_status_type: A name that matches the name of a StatusElement
            p_coordinate: The position for the status bar

        Returns:
            The new StatusElement

        Raises:
            ValueError: If no class name matches the p_status_type
        """
        if p_status_type.lower() == "health":
            return HealthBar(p_coordinate)
        elif p_status_type.lower() == "energy":
            return EnergyBar(p_coordinate)
        else:
            raise ValueError(p_status_type)
